// dcodedoc.cpp : implementation of the CDcodeDoc class
//
                                      
#include "stdafx.h"
#include "mainfrm.h" 
#include "dcodedoc.h"
#include "dcode.h"
#include "customap.h"
#include "enlgdial.h"
#include "meterbar.h"
#include <stdlib.h> 

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

extern char far headmsg[]; 

#define TOPLAYER		1
#define BOTTOMLAYER		6
#define MULTILAYER		13

/////////////////////////////////////////////////////////////////////////////
// CDcodeDoc

IMPLEMENT_DYNCREATE(CDcodeDoc, CDocument)

BEGIN_MESSAGE_MAP(CDcodeDoc, CDocument)
	//{{AFX_MSG_MAP(CDcodeDoc)
	ON_COMMAND(ID_ADD_APER, OnAddAper)
	ON_COMMAND(ID_MAKE_TABLE, OnMakeTable)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	ON_UPDATE_COMMAND_UI(ID_MAKE_TABLE, OnUpdateMakeTable)
	ON_COMMAND(ID_SET_ENLG, OnSetEnlg)
	ON_COMMAND(ID_CLEAR_TABLE, OnClearTable)
	ON_UPDATE_COMMAND_UI(ID_CLEAR_TABLE, OnUpdateClearTable)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDcodeDoc construction/destruction

CDcodeDoc::CDcodeDoc()
{
	TRACE("\nEntering CDcodeDoc constructor");
	m_pInputFile = NULL; 
	m_pOutputFile = NULL;
	m_pApList = new CApertureObList;
	m_nPGEnlg = 10;
	m_nSMEnlg = 10;
	m_nPMEnlg = 10; 
	m_bFills = FALSE;
}

CDcodeDoc::~CDcodeDoc()
{   
	TRACE("\nEntering Destructor");
	
	if(m_pInputFile != NULL)
		TidyUp(m_pInputFile);	
	if(m_pOutputFile != NULL)
		TidyUp(m_pOutputFile);	
		 
	if(!(m_pApList->IsEmpty()))
	{
		TRACE("\nDestructor - Removing all Apertures from list");
		POSITION pos;
					 
		for(pos = m_pApList->GetHeadPosition(); pos != NULL; )
		{ 
			TRACE("\nCurrent list entries = %d", CApertureObject::nApertureCount);
			delete (CApertureObject*) (m_pApList->GetNext(pos));  // delete objects in list 
		}
		m_pApList->RemoveAll();
		ASSERT(m_pApList->IsEmpty());
	}
	else
		TRACE("\nDestructor - List already empty"); 
		
	delete m_pApList;
}

BOOL CDcodeDoc::OnNewDocument()
{
	TRACE("\nEntering OnNewDocument handler");
		
	if (!CDocument::OnNewDocument())
		return FALSE; 
		
	if(AfxMessageBox("Do you want to load standard aperture list?", 
							MB_YESNO | MB_ICONQUESTION, 0) == IDYES)
	{
		if(!LoadStdApertures())
			return FALSE; 
	}
		
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CDcodeDoc serialization

void CDcodeDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDcodeDoc diagnostics

#ifdef _DEBUG
void CDcodeDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDcodeDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDcodeDoc commands

void CDcodeDoc::OnAddAper()
{
	TRACE("\nEntering OnAddAper handler");
	
	CCustomAper cca;
	if(cca.DoModal() == IDOK)
	{
		SHAPES nShape; 
		TYPES nType; 
		int nXSize, nYSize;
		
		nShape = (SHAPES) (cca.m_nShape + 1);
		nType = (TYPES) (cca.m_nType + 1);
		nXSize = cca.m_nXSize;
		nYSize = cca.m_nYSize;
		CApertureObject& cao = *(new CApertureObject(nShape, nType, nXSize, nYSize));
		if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
			m_pApList->AddHead(&cao);        // add to list
		else
			delete &cao; 
		if(cca.m_bPad)
		{
			if(m_nPGEnlg)
			{
				CApertureObject& cao = *(new CApertureObject(nShape, nType, 
									          nXSize + 2 * m_nPGEnlg, nYSize + 2 * m_nPGEnlg));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao;
			}
				
			if((m_nSMEnlg) && (m_nSMEnlg != m_nPGEnlg))
			{
				CApertureObject& cao = *(new CApertureObject(nShape, nType, 
										       nXSize + 2 * m_nSMEnlg, nYSize + 2 * m_nSMEnlg)); 
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao;
			} 
				
			if((m_nPMEnlg) && (m_nPMEnlg != m_nPGEnlg) && (m_nPMEnlg != m_nSMEnlg) 
			&& (nXSize + 2 * m_nPMEnlg > 0) && (nYSize + 2 * m_nPMEnlg > 0))
			{
				CApertureObject& cao = *(new CApertureObject(nShape, nType, 
										       nXSize + 2 * m_nPMEnlg, nYSize + 2 * m_nPMEnlg));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao;
			} 
		}	
	} 
	UpdateAllViews(NULL);
	return;
}

void CDcodeDoc::OnMakeTable()
{
	TRACE("\nEntering OnMakeTable handler"); 
	ASSERT_VALID(m_pInputFile);
	m_pInputFile->SeekToBegin();
	char buffer[41];
	CString cs;
	int count = 0;
	int p1, p2, p3, p4, p5, p6, p7;	// storage for other data parameters
	int nShape, nXSize, nYSize, nLayer;
	int nNewPercent, nOldPercent;
	CMeterBar* pMeterBar; 
	long linecount = 0;
	
	nNewPercent = nOldPercent = 0;

	DWORD wFileLength = m_pInputFile->GetLength();
	DWORD wFilePos = m_pInputFile->GetPosition();
	
	pMeterBar = &(((CMainFrame*)AfxGetMainWnd())->m_wndStatusBar); 
	pMeterBar->SetProgressBar(0.0);

	do
	{
		cs = CString(m_pInputFile->ReadString(buffer, 40));    // read next line
		linecount++;
		if(*buffer == '\0')
			break;
		if(cs.Find("ENDPCB") >= 0)
			break;
		if(count)
			count--;
		else
		{
			if(cs.Left(4) == "COMP")
				count = 6;
			else if((cs.Left(2) == "CA") || (cs.Left(2) == "FA"))    // arcs
			{
				m_pInputFile->ReadString(buffer, 40);
				linecount++;
				sscanf(buffer, "%d %d %d %d %d %d", &p1, &p2, &p3, &p4, &nXSize, &p6); 
				CApertureObject& cao = *(new CApertureObject(CIRCULAR, MULTI, nXSize, nXSize));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list 
				else
					delete &cao;
			} 
			else if((cs.Left(2) == "CV") || (cs.Left(2) == "FV"))    // vias
			{
				m_pInputFile->ReadString(buffer, 40);
				linecount++;
				sscanf(buffer, "%d %d %d %d", &p1, &p2, &nXSize, &p4);
				CApertureObject& cao = *(new CApertureObject(CIRCULAR, MULTI, nXSize, nXSize));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao; 
					
				if(m_nPGEnlg)
				{
					CApertureObject& cao = *(new CApertureObject(CIRCULAR, MULTI, 
										          nXSize + 2 * m_nPGEnlg, nXSize + 2 * m_nPGEnlg));
					if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
						m_pApList->AddHead(&cao);        // add to list
					else
						delete &cao;
				}
			}
			else if((cs.Left(2) == "CT") || (cs.Left(2) == "FT"))     // tracks
			{		
				m_pInputFile->ReadString(buffer, 40);
				linecount++;
				sscanf(buffer, "%d %d %d %d %d %d %d", &p1, &p2, &p3, &p4, &nXSize, &p6, &p7);
				CApertureObject& cao = *(new CApertureObject(CIRCULAR, MULTI, nXSize, nXSize));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao;
			}
			else if((cs.Left(2) == "CS") || (cs.Left(2) == "FS"))      // strings
			{
				m_pInputFile->ReadString(buffer, 40);
				linecount++;
				sscanf(buffer, "%d %d %d %d %d %d", &p1, &p2, &p3, &p4, &nXSize, &p6);
				CApertureObject& cao = *(new CApertureObject(CIRCULAR, MULTI, nXSize, nXSize));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao;
					
				m_pInputFile->ReadString(buffer, 40);    // throw away useless line
				linecount++;
			}
/*
#define TOPLAYER		1
#define BOTTOMLAYER		6
#define MULTILAYER		13
*/
		else if((cs.Left(2) == "CP") || (cs.Left(2) == "FP"))
			{
				m_pInputFile->ReadString(buffer, 40);
				linecount++;
				sscanf(buffer, "%d %d %d %d %d %d %d %d", 
				       &p1, &p2, &nXSize, &nYSize, &nShape, &p6, &p7, &nLayer);
				       
				CApertureObject& cao = *(new CApertureObject((SHAPES) nShape, MULTI, nXSize, nYSize));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao;
				
				if(m_nPGEnlg && (nLayer == MULTILAYER))
				{
					CApertureObject& cao = *(new CApertureObject((SHAPES) nShape, MULTI, nXSize + 2 * m_nPGEnlg, nYSize + 2 * m_nPGEnlg));
					if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
						m_pApList->AddHead(&cao);        // add to list
					else
						delete &cao;
				}
				
				if(m_nSMEnlg)
				{
					CApertureObject& cao = *(new CApertureObject((SHAPES) nShape, MULTI, nXSize + 2 * m_nSMEnlg, nYSize + 2 * m_nSMEnlg)); 
					if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
						m_pApList->AddHead(&cao);        // add to list
					else
						delete &cao;
				} 
				
				if((m_nPMEnlg) && (m_nPMEnlg != m_nPGEnlg) && (m_nPMEnlg != m_nSMEnlg) && (nXSize + 2 * m_nPMEnlg > 0) && (nYSize + 2 * m_nPMEnlg > 0))
				{
					CApertureObject& cao = *(new CApertureObject((SHAPES) nShape, MULTI, nXSize + 2 * m_nPMEnlg, nYSize + 2 * m_nPMEnlg));
					if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
						m_pApList->AddHead(&cao);        // add to list
					else
						delete &cao;
				} 
				m_pInputFile->ReadString(buffer, 40);    // throw away useless line
				linecount++;
			}
			else if(m_bFills && ((cs.Left(2) == "CF") || (cs.Left(2) == "FF")))   // fills
			{
				m_pInputFile->ReadString(buffer, 40);
				linecount++;
				sscanf(buffer, "%d %d %d %d %d", &p1, &p2, &p3, &p4, &p5);
				nXSize = abs(p3 - p1);
				nYSize = abs(p4 - p2);
				if(nXSize > 500)
					nXSize = 500;
				if(nYSize > 500)
					nYSize = 500;
				CApertureObject& cao = *(new CApertureObject(RECTANGULAR, MULTI, nXSize, nYSize));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao;
			}
		} 
		DWORD wFilePos = m_pInputFile->GetPosition(); 
		nNewPercent = (int) min(100,((100 * wFilePos)/wFileLength));
		if((nNewPercent - 1) >= nOldPercent)
		{
			nOldPercent = nNewPercent;	
			pMeterBar->SetProgressBar((double) nOldPercent / 100.0);
		}
	} while(1);

	pMeterBar->SetProgressBar(0.0);
	UpdateAllViews(NULL);
	return;
}

void CDcodeDoc::OnFileSave()
{
	TRACE("\nEntering OnFileSave handler\n");   

   	if(m_pApList->IsEmpty())
   	{
   		AfxMessageBox("There are no apertures in list!");
		return;     // return if file open failed
   	}
	
	static char BASED_CODE fileDialogFilter[] = "Aperture files (*.apt)|*.apt|All Files (*.*)|*.*||"; 
	static char BASED_CODE fileDialogExt[] = "apt";
	
	CFileDialog fileDialog(FALSE, fileDialogExt, NULL, OFN_HIDEREADONLY, fileDialogFilter); 
	
	CString sName = m_sInputFileName.SpanExcluding(".") + ".apt"; 
	sName.MakeLower();
/*
	char buf[256];
	strncpy(buf, sName.GetBuffer(256), sName.GetLength());
	sName.ReleaseBuffer(-1);
	buf[sName.GetLength()] = '\0';
	fileDialog.m_ofn.lpstrFile = buf;	
*/
	if(fileDialog.DoModal() != IDOK) 
		return; 

	CFileException exception;
	BOOL status;
		
	if(m_pOutputFile)           			// if the file is somehow already open
		TidyUp(m_pOutputFile);

	m_pOutputFile = new CStdioFile;    		// otherwise, create new object   
	CString filepath = fileDialog.GetPathName();
                          
	status = m_pOutputFile->Open((const char*)filepath, CFile::modeWrite | CFile::modeCreate, &exception); 
		
	if(!status)        // file not opened
	{
		char s[100];
		sprintf(s, "Error opening file for writing.  Code: %d", exception.m_cause);
		AfxMessageBox(s, MB_OK, 0); 
	}
   	else 
   	{
	   	ASSERT_VALID(m_pOutputFile);
	   	m_pOutputFile->WriteString(headmsg);
	   	m_pApList->WriteAllApertures(m_pOutputFile);	
	}
	TidyUp(m_pOutputFile); 
//	ASSERT(m_pOutputFile == NULL);
	return;
}

void CDcodeDoc::TidyUp(CStdioFile*& pFile)
{
	TRACE("Tidying up\n");
	if(pFile == NULL)
		return;
	ASSERT_VALID(pFile);
	pFile->Close();    // tidy up
	delete pFile; 
	pFile = NULL;   
	ASSERT(pFile == NULL);
}

void CDcodeDoc::OnFileOpen()
{
	TRACE("Entering OnFileOpen handler\n");

	if(m_pInputFile != NULL) 
	{
		TRACE("\nFile already open");
		if(AfxMessageBox("You have a PCB file open already.  OK to close?", MB_YESNO, 0) == IDNO)
		{
			TRACE("\nLeaving OnFileOpen handler");
			return;
		}
		else
		{
			TRACE("\nClosing input file");  
			TidyUp(m_pInputFile);
		}
	}  
	
	static char BASED_CODE fileDialogFilter[] =
	"PCB files (*.pcb)|*.pcb|All Files (*.*)|*.*||"; 
	
	const char fileDialogExt[] = "pcb"; 
	
	CFileDialog fileDialog( TRUE, 
		fileDialogExt, NULL,
		OFN_FILEMUSTEXIST, fileDialogFilter );
		 
	if( fileDialog.DoModal() == IDOK )
	{   
		CFileException exception;
		BOOL status;
		
		m_pInputFile = new CStdioFile;
		                              
		status = m_pInputFile->Open((const char*)fileDialog.GetPathName(), 
											CFile::modeRead, &exception); 
		if(!status)        // file not opened
		{
			char s[100];
			
			sprintf(s, "Error opening file for reading.  Code: %d", exception.m_cause);
			AfxMessageBox(s, MB_OK, 0);
			TidyUp(m_pInputFile);
			return;
		}
		else               // file opened, but is it correct???
		{  
			char buffer[81];
			CString cs(m_pInputFile->ReadString(buffer, 80));    // read first line
			
			if(!((cs == "PCB FILE 4\n") || (cs == "PCB FILE 5\n")))  // check first line
			{
				char s[100];
				
				sprintf(s, "Input file is not an Easytrax or Autotrax file.");
				
				AfxMessageBox(s, MB_OK | MB_ICONEXCLAMATION, 0);
				TRACE("\nFirst line is %s\n",(const char*)cs);
				
				TidyUp(m_pInputFile);
			}
			else
				m_sInputFileName = fileDialog.GetFileName();
     	} 
   }
	return;
}

void CDcodeDoc::OnSetEnlg()
{
	TRACE("\nEntering OnSetEnlg handler"); 

	CEnlgDialog enld;

	enld.m_nPMEnlg = m_nPMEnlg;
	enld.m_nPGEnlg = m_nPGEnlg;
	enld.m_nSMEnlg = m_nSMEnlg; 
	enld.m_bFills = m_bFills;
	
	enld.DoModal();
	
	m_nPMEnlg = enld.m_nPMEnlg;
	m_nPGEnlg = enld.m_nPGEnlg;
	m_nSMEnlg = enld.m_nSMEnlg;
	m_bFills = enld.m_bFills;
	return;
	
}

void CDcodeDoc::OnClearTable()
{
	if(!(m_pApList->IsEmpty()))
	{
		TRACE("\nClear Table - Removing all apertures from list");
		POSITION pos;
					 
		for(pos = m_pApList->GetHeadPosition(); pos != NULL; )
		{ 
			TRACE("\nCurrent list entries = %d", CApertureObject::nApertureCount);
			delete (CApertureObject*) (m_pApList->GetNext(pos));  // delete objects in list 
		}
		m_pApList->RemoveAll();
		ASSERT(m_pApList->IsEmpty()); 
		if(AfxMessageBox("Do you want to reload the standard aperture list?", 
								MB_YESNO | MB_ICONQUESTION, 0) == IDYES)
			LoadStdApertures();
	}
	else
		TRACE("\nClear Table - List already empty");
		
	UpdateAllViews(NULL); 
	return;
}

BOOL CDcodeDoc::LoadStdApertures()
{
	static char BASED_CODE stdApFilter[] =
	"Aperture files (*.std)|*.std|All Files (*.*)|*.*||"; 
	
	const char stdApExt[] = "std"; 
	
	CFileDialog stdApDialog( TRUE, 
		stdApExt, NULL,
		OFN_FILEMUSTEXIST, stdApFilter );
		 
	if( stdApDialog.DoModal() == IDOK )
	{   
		CFileException exception;
		BOOL status;
			
		CStdioFile stdapfile;
			                              
		status = stdapfile.Open((const char*)stdApDialog.GetPathName(), CFile::modeRead, &exception);
		if(!status)        // file not opened
		{
			char s[100];
				
			sprintf(s, "Error opening standard aperture file.  Code: %d", exception.m_cause);
			AfxMessageBox(s, MB_OK, 0);
				 
			return FALSE;     // return if file open failed
		}
		else
		{
			char szShape[50], szXSize[50], szYSize[50], szHoleSize[50], szType[50];
			int nXSize, nYSize;
			SHAPES nShape;
			TYPES nType;
			char buffer[100];
				
			TRACE("\nLoading standard apertures"); 
			while(stdapfile.ReadString(buffer, sizeof(buffer) - 1))
			{
				sscanf(buffer, "%s %s %s %s %s", szShape, szXSize, szYSize, szHoleSize, szType);
				nXSize = atoi(szXSize);
				nYSize = atoi(szYSize);
				CString csShape(szShape);
				if			(csShape == "RECTANGULAR") nShape = RECTANGULAR;
				else if	(csShape == "OCTAGONAL") 	nShape = OCTAGONAL;
				else if	(csShape == "ROUNDRECT") 	nShape = ROUNDRECT;
				else if	(csShape == "CROSSHAIR") 	nShape = CROSSHAIR;
				else if	(csShape == "MOIRE") 		nShape = MOIRE;
				else 											nShape = CIRCULAR;
				CString csType(szType);
				if			(csType == "LINE") 			nType = LINE;
				else if	(csType == "FLASH") 			nType = FLASH;
				else 											nType = MULTI;
					
				CApertureObject& cao = *(new CApertureObject(nShape, nType, nXSize, nYSize));
				if(!(m_pApList->SearchList(cao)))   // doesn't already exixt
					m_pApList->AddHead(&cao);        // add to list
				else
					delete &cao; 
			}
			stdapfile.Close();    // tidy up
	   } 
	}
	return TRUE;
}

void CDcodeDoc::OnUpdateFileSave(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!(m_pApList->IsEmpty()));
	return;
}

void CDcodeDoc::OnUpdateMakeTable(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(m_pInputFile != NULL);
	return;
}

void CDcodeDoc::OnUpdateClearTable(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(!(m_pApList->IsEmpty()));
	return;
}
